@@ -1,5 +1,4 @@
-#! /usr/bin/env python
-from __future__ import print_function
+#!/usr/bin/env python3
 
 """
 IKFast Plugin Generator for MoveIt
@@ -63,7 +62,7 @@ except ImportError:
         pass
 
     def get_pkg_dir(pkg_name):
-        raise InvalidROSPkgException("Failed to locate ROS package {}".format(pkg_name))
+        raise InvalidROSPkgException(f"Failed to locate ROS package {pkg_name}")
 
 
 # Package containing this file
@@ -127,16 +126,16 @@ def populate_optional(args):
 
 def print_args(args):
     print("Creating IKFastKinematicsPlugin with parameters: ")
-    print(" robot_name:           %s" % args.robot_name)
-    print(" base_link_name:       %s" % args.base_link_name)
-    print(" eef_link_name:        %s" % args.eef_link_name)
-    print(" planning_group_name:  %s" % args.planning_group_name)
-    print(" ikfast_plugin_pkg:    %s" % args.ikfast_plugin_pkg)
-    print(" ikfast_output_path:   %s" % args.ikfast_output_path)
-    print(" search_mode:          %s" % args.search_mode)
-    print(" srdf_filename:        %s" % args.srdf_filename)
-    print(" robot_name_in_srdf:   %s" % args.robot_name_in_srdf)
-    print(" moveit_config_pkg:    %s" % args.moveit_config_pkg)
+    print(f" robot_name:           {args.robot_name}")
+    print(f" base_link_name:       {args.base_link_name}")
+    print(f" eef_link_name:        {args.eef_link_name}")
+    print(f" planning_group_name:  {args.planning_group_name}")
+    print(f" ikfast_plugin_pkg:    {args.ikfast_plugin_pkg}")
+    print(f" ikfast_output_path:   {args.ikfast_output_path}")
+    print(f" search_mode:          {args.search_mode}")
+    print(f" srdf_filename:        {args.srdf_filename}")
+    print(f" robot_name_in_srdf:   {args.robot_name_in_srdf}")
+    print(f" moveit_config_pkg:    {args.moveit_config_pkg}")
     print("")
 
 
@@ -160,7 +159,7 @@ def validate_openrave_version(args):
                 if line_search:
                     solver_version = int(line_search.group(1), 0) & ~0x10000000
                 break
-    print("Found source code generated by IKFast version %s" % str(solver_version))
+    print(f"Found source code generated by IKFast version {solver_version}")
 
     # Chose template depending on IKFast version
     if solver_version >= 56:
@@ -181,8 +180,7 @@ def create_ikfast_package(args):
     except InvalidROSPkgException:
         args.ikfast_plugin_pkg_path = os.path.abspath(args.ikfast_plugin_pkg)
         print(
-            "Failed to find package: %s. Will create it in %s."
-            % (args.ikfast_plugin_pkg, args.ikfast_plugin_pkg_path)
+            f"Createing new package {args.ikfast_plugin_pkg} it in {args.ikfast_plugin_pkg_path}."
         )
         # update pkg name to basename of path
         args.ikfast_plugin_pkg = os.path.basename(args.ikfast_plugin_pkg_path)
@@ -207,13 +205,13 @@ def create_ikfast_package(args):
         root.append(xmlElement("license", text="BSD"))
         user_name = getuser()
         root.append(
-            xmlElement("maintainer", email="%s@todo.todo" % user_name, text=user_name)
+            xmlElement("maintainer", email=f"{user_name}@todo.todo", text=user_name)
         )
         root.append(xmlElement("buildtool_depend", text="catkin"))
         etree.ElementTree(root).write(
             pkg_xml_path, xml_declaration=True, pretty_print=True, encoding="UTF-8"
         )
-        print("Created package.xml at: '%s'" % pkg_xml_path)
+        print(f"Created package.xml at: '{pkg_xml_path}'")
 
 
 def find_template_dir():
@@ -225,7 +223,7 @@ def find_template_dir():
             get_pkg_dir(plugin_gen_pkg), "ikfast_kinematics_plugin/templates"
         )
     except InvalidROSPkgException:
-        raise Exception("Can't find package %s" % plugin_gen_pkg)
+        raise Exception(f"Can't find package {plugin_gen_pkg}")
 
 
 def update_ikfast_package(args):
@@ -245,9 +243,8 @@ def update_ikfast_package(args):
 
     if not os.path.exists(solver_file_path):
         raise Exception(
-            "Failed to copy IKFast source code from '%s' to '%s'\n"
+            f"Failed to copy IKFast source code from '{args.ikfast_output_path}' to '{solver_file_path}'\n"
             "Manually copy the source file generated by IKFast to this location and re-run"
-            % (args.ikfast_output_path, solver_file_path)
         )
     # Remember ikfast solver file for update of MoveIt package
     args.ikfast_output_path = solver_file_path
@@ -301,13 +298,7 @@ def update_ikfast_package(args):
         base_class_type="kinematics::KinematicsBase",
     )
     desc = etree.SubElement(cl, "description")
-    desc.text = (
-        "IKFast{template} plugin for closed-form kinematics of {robot} {group}".format(
-            template=args.template_version,
-            robot=args.robot_name,
-            group=args.planning_group_name,
-        )
-    )
+    desc.text = f"IKFast{args.template_version} plugin for closed-form kinematics of {args.robot_name} {args.planning_group_name}"
 
     # Write plugin definition to file
     plugin_file_name = ik_library_name + "_description.xml"
@@ -315,7 +306,7 @@ def update_ikfast_package(args):
     etree.ElementTree(plugin_def).write(
         plugin_file_path, xml_declaration=True, pretty_print=True, encoding="UTF-8"
     )
-    print("Created plugin definition at  '%s'" % plugin_file_path)
+    print(f"Created plugin definition at  '{plugin_file_path}'")
 
     # Create CMakeLists file
     replacements.update(dict(_LIBRARY_NAME_=ik_library_name))
@@ -366,7 +357,7 @@ def update_ikfast_package(args):
     etree.ElementTree(package_xml).write(
         package_file_name, xml_declaration=True, pretty_print=True, encoding="UTF-8"
     )
-    print("Wrote package.xml at  '%s'" % package_file_name)
+    print(f"Wrote package.xml at  '{package_file_name}'")
 
     # Create a script for easily updating the plugin in the future in case the plugin needs to be updated
     easy_script_file_path = args.ikfast_plugin_pkg_path + "/update_ikfast_plugin.sh"
@@ -415,64 +406,63 @@ def update_ikfast_package(args):
             + "  $ikfast_output_path\n"
         )
 
-    print("Created update plugin script at '%s'" % easy_script_file_path)
+    print(f"Created update plugin script at '{easy_script_file_path}'")
 
 
 def update_moveit_package(args):
     try:
         moveit_config_pkg_path = get_pkg_dir(args.moveit_config_pkg)
     except InvalidROSPkgException:
-        raise Exception("Failed to find package: " + args.moveit_config_pkg)
+        raise RuntimeError(f"Failed to find package: {args.moveit_config_pkg}")
 
+    # SRDF sanity checks
     try:
         srdf_file_name = moveit_config_pkg_path + "/config/" + args.srdf_filename
         srdf = etree.parse(srdf_file_name).getroot()
-    except IOError:
-        raise Exception("Failed to find SRDF file: " + srdf_file_name)
-    except etree.XMLSyntaxError as err:
-        raise Exception(
-            "Failed to parse xml in file: %s\n%s" % (srdf_file_name, err.msg)
-        )
 
-    if args.robot_name_in_srdf != srdf.get("name"):
-        raise Exception(
-            "Robot name in srdf ('%s') doesn't match expected name ('%s')"
-            % (srdf.get("name"), args.robot_name_in_srdf)
-        )
-
-    groups = srdf.findall("group")
-    if len(groups) < 1:
-        raise Exception("No planning groups are defined in the SRDF")
-
-    planning_group = None
-    for group in groups:
-        if group.get("name").lower() == args.planning_group_name.lower():
-            planning_group = group
-
-    if planning_group is None:
-        raise Exception(
-            "Planning group '%s' not defined in the SRDF. Available groups: \n%s"
-            % (
-                args.planning_group_name,
-                ", ".join([group_name.get("name") for group_name in groups]),
+        robot_name = srdf.get("name")
+        if args.robot_name_in_srdf != robot_name:
+            raise RuntimeWarning(
+                f"Robot name in srdf ('{robot_name}') doesn't match expected name ('{args.robot_name_in_srdf}')"
             )
-        )
+
+        groups = [g.get("name") for g in srdf.findall("group")]
+        if args.planing_group_name not in groups:
+            raise RuntimeWarning(
+                f"Planning group '{args.planning_group_name}' not defined in the SRDF."
+                " Available groups: \n" + ", ".join(groups)
+            )
+
+    except IOError:
+        print(f"Failed to find SRDF file: {srdf_file_name}")
+    except etree.XMLSyntaxError as err:
+        print(f"Failed to parse xml in file: {srdf_file_name}\n{err.msg}")
 
     # Modify kinematics.yaml file
     kin_yaml_file_name = moveit_config_pkg_path + "/config/kinematics.yaml"
     with open(kin_yaml_file_name, "r") as f:
         kin_yaml_data = yaml.safe_load(f)
 
-    kin_yaml_data[args.planning_group_name]["kinematics_solver"] = args.plugin_name
+    try:
+        kin_yaml_data[args.planning_group_name]["kinematics_solver"] = args.plugin_name
+    except KeyError:
+        # create new group entry
+        print(
+            f"Creating new planning group entry in kinematics.yaml for '{args.planning_group_name}'"
+        )
+        kin_yaml_data[args.planning_group_name] = dict(
+            kinematics_solver=args.plugin_name
+        )
+
     with open(kin_yaml_file_name, "w") as f:
         yaml.dump(kin_yaml_data, f, default_flow_style=False)
 
-    print("Modified kinematics.yaml at '%s'" % kin_yaml_file_name)
+    print(f"Modified kinematics.yaml at '{kin_yaml_file_name}'")
 
 
 def copy_file(src_path, dest_path, description, replacements=None):
     if not os.path.exists(src_path):
-        raise Exception("Can't find %s at '%s'" % (description, src_path))
+        raise Exception(f"Can't find {description} at '{src_path}'")
 
     if replacements is None:
         replacements = dict()
@@ -486,7 +476,7 @@ def copy_file(src_path, dest_path, description, replacements=None):
 
     with open(dest_path, "w") as f:
         f.write(content)
-    print("Created %s at '%s'" % (description, dest_path))
+    print(f"Created {description} at '{dest_path}'")
 
 
 def main():
@@ -500,8 +490,14 @@ def main():
     update_ikfast_package(args)
     try:
         update_moveit_package(args)
-    except Exception as e:
-        print("Failed to update MoveIt package:\n" + str(e))
+    except Exception as ex:
+        print(
+            f"""Failed to update MoveIt package:
+{ex}
+Update your kinematics.yaml manually to include the following configuration:
+{args.planning_group_name}:
+  kinematics_solver: {args.plugin_name}"""
+        )
 
 
 if __name__ == "__main__":
